@@ -20,14 +20,14 @@ from firebolt.utils.exception import (
     InterfaceError,
 )
 from firebolt.utils.urls import (
-    ACCOUNT_ENGINE_BY_NAME_URL,
+    ACCOUNT_ENGINE_ID_BY_NAME_URL,
     ACCOUNT_ENGINE_URL,
     ACCOUNT_ENGINE_URL_BY_DATABASE_NAME,
 )
 from firebolt.utils.usage_tracker import get_user_agent_header
 from firebolt.utils.util import fix_url_schema
 
-DEFAULT_TIMEOUT_SECONDS: int = 5
+DEFAULT_TIMEOUT_SECONDS: int = 60
 KEEPALIVE_FLAG: int = 1
 KEEPIDLE_RATE: int = 60  # seconds
 AUTH_CREDENTIALS_DEPRECATION_MESSAGE = """ Passing connection credentials
@@ -56,32 +56,37 @@ async def _resolve_engine_url(
         base_url=api_endpoint,
         account_name=account_name,
         api_endpoint=api_endpoint,
+        timeout=Timeout(DEFAULT_TIMEOUT_SECONDS),
     ) as client:
+        account_id = await client.account_id
+        url = ACCOUNT_ENGINE_ID_BY_NAME_URL.format(account_id=account_id)
         try:
-            account_id = await client.account_id
             response = await client.get(
-                url=ACCOUNT_ENGINE_BY_NAME_URL.format(account_id=account_id),
+                url=url,
                 params={"engine_name": engine_name},
             )
             response.raise_for_status()
             engine_id = response.json()["engine_id"]["engine_id"]
-            response = await client.get(
-                url=ACCOUNT_ENGINE_URL.format(
-                    account_id=account_id, engine_id=engine_id
-                ),
-            )
+            url = ACCOUNT_ENGINE_URL.format(account_id=account_id, engine_id=engine_id)
+            response = await client.get(url=url)
             response.raise_for_status()
             return response.json()["engine"]["endpoint"]
         except HTTPStatusError as e:
             # Engine error would be 404.
             if e.response.status_code != 404:
-                raise InterfaceError(f"Unable to retrieve engine endpoint: {e}.")
+                raise InterfaceError(
+                    f"Error {e.__class__.__name__}: Unable to retrieve engine "
+                    f"endpoint {url}."
+                )
             # Once this is point is reached we've already authenticated with
             # the backend so it's safe to assume the cause of the error is
             # missing engine.
             raise FireboltEngineError(f"Firebolt engine {engine_name} does not exist.")
         except (JSONDecodeError, RequestError, RuntimeError, HTTPStatusError) as e:
-            raise InterfaceError(f"Unable to retrieve engine endpoint: {e}.")
+            raise InterfaceError(
+                f"Error {e.__class__.__name__}: "
+                f"Unable to retrieve engine endpoint {url}."
+            )
 
 
 async def _get_database_default_engine_url(
@@ -95,6 +100,7 @@ async def _get_database_default_engine_url(
         base_url=api_endpoint,
         account_name=account_name,
         api_endpoint=api_endpoint,
+        timeout=Timeout(DEFAULT_TIMEOUT_SECONDS),
     ) as client:
         try:
             account_id = await client.account_id
@@ -356,7 +362,7 @@ class BaseConnection:
 
     @property
     def closed(self) -> bool:
-        """`True if connection is closed; False otherwise."""
+        """`True` if connection is closed; `False` otherwise."""
         return self._is_closed
 
     def _remove_cursor(self, cursor: Cursor) -> None:
