@@ -5,7 +5,6 @@ import re
 import time
 from enum import Enum
 from functools import wraps
-from inspect import cleandoc
 from types import TracebackType
 from typing import (
     TYPE_CHECKING,
@@ -36,6 +35,7 @@ from firebolt.async_db._types import (
 from firebolt.async_db.util import is_db_available, is_engine_running
 from firebolt.client import AsyncClient
 from firebolt.utils.exception import (
+    AsyncExecutionUnavailableError,
     CursorClosedError,
     DataError,
     EngineNotRunningError,
@@ -61,6 +61,19 @@ class CursorState(Enum):
     CLOSED = 4
 
 
+class QueryStatus(Enum):
+    """Enumeration of query responses on server-side async queries."""
+
+    RUNNING = 1
+    ENDED_SUCCESSFULLY = 2
+    ENDED_UNSUCCESSFULLY = 3
+    NOT_READY = 4
+    STARTED_EXECUTION = 5
+    PARSE_ERROR = 6
+    CANCELED_EXECUTION = 7
+    EXECUTION_ERROR = 8
+
+
 class Statistics(BaseModel):
     """
     Class for query execution statistics.
@@ -88,12 +101,10 @@ def check_not_closed(func: Callable) -> Callable:
 
 
 def check_query_executed(func: Callable) -> Callable:
-    cleandoc(
-        """
-        (Decorator) ensure that some query has been executed before
-        calling cursor method.
-        """
-    )
+    """
+    (Decorator) ensure that some query has been executed before
+    calling cursor method.
+    """
 
     @wraps(func)
     def inner(self: Cursor, *args: Any, **kwargs: Any) -> Any:
@@ -119,6 +130,7 @@ class BaseCursor:
         "_row_sets",
         "_next_set_idx",
         "_set_parameters",
+        "_query_id",
     )
 
     default_arraysize = 1
@@ -143,6 +155,7 @@ class BaseCursor:
         self._rowcount = -1
         self._idx = 0
         self._next_set_idx = 0
+        self._query_id = ""
         self._reset()
 
     def __del__(self) -> None:
@@ -177,6 +190,12 @@ class BaseCursor:
         """The number of rows produced by last query."""
         return self._rowcount
 
+    @property  # type: ignore
+    @check_not_closed
+    def query_id(self) -> str:
+        """The query id of a query executed asynchronously."""
+        return self._query_id
+
     @property
     def arraysize(self) -> int:
         """Default number of rows returned by fetchmany."""
@@ -194,12 +213,10 @@ class BaseCursor:
     @property
     def closed(self) -> bool:
         """True if connection is closed, False otherwise."""
-
         return self._state == CursorState.CLOSED
 
     def close(self) -> None:
         """Terminate an ongoing query (if any) and mark connection as closed."""
-
         self._state = CursorState.CLOSED
         # remove typecheck skip  after connection is implemented
         self.connection._remove_cursor(self)  # type: ignore
@@ -267,6 +284,7 @@ class BaseCursor:
         self._idx = 0
         self._row_sets = []
         self._next_set_idx = 0
+        self._query_id = ""
 
     def _row_set_from_response(
         self, response: Response
@@ -276,12 +294,11 @@ class BaseCursor:
         Optional[Statistics],
         Optional[List[List[RawColType]]],
     ]:
-        """Fetch information about executed query from http response"""
+        """Fetch information about executed query from http response."""
 
         # Empty response is returned for insert query
         if response.headers.get("content-length", "") == "0":
             return (-1, None, None, None)
-
         try:
             # Skip parsing floats to properly parse them later
             query_data = response.json(parse_float=str)
@@ -313,52 +330,102 @@ class BaseCursor:
             self._pop_next_set()
 
     async def _api_request(
-        self, query: str, set_parameters: Optional[dict]
+        self,
+        query: Optional[str] = "",
+        parameters: Optional[dict[str, Any]] = {},
+        path: Optional[str] = "",
+        use_set_parameters: Optional[bool] = True,
     ) -> Response:
+        """
+        Query API, return Response object.
+
+        Args:
+            query (str): SQL query
+            parameters (Optional[Sequence[ParameterType]]): A sequence of substitution
+                parameters. Used to replace '?' placeholders inside a query with
+                actual values. Note: In order to "output_format" dict value, it
+                    must be an empty string. If no value not specified,
+                    JSON_OUTPUT_FORMAT will be used.
+            path (str): endpoint suffix, for example "cancel" or "status"
+            use_set_parameters: Optional[bool]: Some queries will fail if additional
+                set parameters are sent. Setting this to False will allow
+                self._set_parameters to be ignored.
+        """
+        if use_set_parameters:
+            parameters = {**(self._set_parameters or {}), **(parameters or {})}
         return await self._client.request(
-            url="/",
+            url=f"/{path}",
             method="POST",
             params={
                 "database": self.connection.database,
-                "output_format": JSON_OUTPUT_FORMAT,
-                **self._set_parameters,
-                **(set_parameters or dict()),
+                **(parameters or dict()),
             },
             content=query,
         )
 
+    async def _validate_set_parameter(self, parameter: SetParameter) -> None:
+        """Validate parameter by executing simple query with it."""
+        if parameter.name == "async_execution":
+            raise AsyncExecutionUnavailableError(
+                "It is not possible to set async_execution using a SET command. "
+                "Instead, pass it as an argument to the execute() or "
+                "executemany() function."
+            )
+        resp = await self._api_request("select 1", {parameter.name: parameter.value})
+        # Handle invalid set parameter
+        if resp.status_code == codes.BAD_REQUEST:
+            raise OperationalError(resp.text)
+        await self._raise_if_error(resp)
+
+        # set parameter passed validation
+        self._set_parameters[parameter.name] = parameter.value
+
+    def _validate_server_side_async_settings(
+        self,
+        parameters: Sequence[Sequence[ParameterType]],
+        queries: List[Union[SetParameter, str]],
+        skip_parsing: bool = False,
+        async_execution: Optional[bool] = False,
+    ) -> None:
+        if async_execution and self._set_parameters.get("use_standard_sql", "1") == "0":
+            raise AsyncExecutionUnavailableError(
+                "It is not possible to execute queries asynchronously if "
+                "use_standard_sql=0."
+            )
+        if parameters and skip_parsing:
+            logger.warning(
+                "Query formatting parameters are provided but skip_parsing "
+                "is specified. They will be ignored."
+            )
+        non_set_queries = 0
+        for query in queries:
+            if type(query) is not SetParameter:
+                non_set_queries += 1
+        if non_set_queries > 1 and async_execution:
+            raise AsyncExecutionUnavailableError(
+                "It is not possible to execute multi-statement "
+                "queries asynchronously."
+            )
+
     async def _do_execute(
         self,
         raw_query: str,
         parameters: Sequence[Sequence[ParameterType]],
-        set_parameters: Optional[Dict] = None,
         skip_parsing: bool = False,
+        async_execution: Optional[bool] = False,
     ) -> None:
         self._reset()
-        if set_parameters is not None:
-            logger.warning(
-                "Passing set parameters as an argument is deprecated. Please run "
-                "a query 'SET <param> = <value>'."
-            )
+        # Allow users to manually skip parsing for performance improvement.
+        queries: List[Union[SetParameter, str]] = (
+            [raw_query] if skip_parsing else split_format_sql(raw_query, parameters)
+        )
         try:
-
-            if parameters and skip_parsing:
-                logger.warning(
-                    "Query formatting parameters are provided with skip_parsing."
-                    " They will be ignored."
-                )
-
-            # Allow users to manually skip parsing for performance improvement
-            queries: List[Union[SetParameter, str]] = (
-                [raw_query] if skip_parsing else split_format_sql(raw_query, parameters)
-            )
-
             for query in queries:
 
                 start_time = time.time()
-                # our CREATE EXTERNAL TABLE queries currently require credentials,
+                # Our CREATE EXTERNAL TABLE queries currently require credentials,
                 # so we will skip logging those queries.
-                # https://docs.firebolt.io/sql-reference/commands/ddl-commands#create-external-table
+                # https://docs.firebolt.io/sql-reference/commands/create-external-table.html
                 if isinstance(query, SetParameter) or not re.search(
                     "aws_key_id|credentials", query, flags=re.IGNORECASE
                 ):
@@ -372,19 +439,35 @@ class BaseCursor:
                     Optional[List[List[RawColType]]],
                 ] = (-1, None, None, None)
                 if isinstance(query, SetParameter):
-                    # Validate parameter by executing simple query with it
-                    resp = await self._api_request(
-                        "select 1", {query.name: query.value}
+                    await self._validate_set_parameter(query)
+                elif async_execution:
+                    self._validate_server_side_async_settings(
+                        parameters,
+                        queries,
+                        skip_parsing,
+                        async_execution,
                     )
-                    # Handle invalid set parameter
-                    if resp.status_code == codes.BAD_REQUEST:
-                        raise OperationalError(resp.text)
-                    await self._raise_if_error(resp)
-
-                    # set parameter passed validation
-                    self._set_parameters[query.name] = query.value
+                    response = await self._api_request(
+                        query,
+                        {
+                            "async_execution": 1,
+                            "advanced_mode": 1,
+                            "output_format": JSON_OUTPUT_FORMAT,
+                        },
+                    )
+                    await self._raise_if_error(response)
+                    if response.headers.get("content-length", "") == "0":
+                        raise OperationalError("No response to asynchronous query.")
+                    resp = response.json()
+                    if "query_id" not in resp or resp["query_id"] == "":
+                        raise OperationalError(
+                            "Invalid response to asynchronous query: missing query_id."
+                        )
+                    self._query_id = resp["query_id"]
                 else:
-                    resp = await self._api_request(query, set_parameters)
+                    resp = await self._api_request(
+                        query, {"output_format": JSON_OUTPUT_FORMAT}
+                    )
                     await self._raise_if_error(resp)
                     row_set = self._row_set_from_response(resp)
 
@@ -392,7 +475,7 @@ class BaseCursor:
 
                 logger.info(
                     f"Query fetched {self.rowcount} rows in"
-                    f" {time.time() - start_time} seconds"
+                    f" {time.time() - start_time} seconds."
                 )
 
             self._state = CursorState.DONE
@@ -406,9 +489,9 @@ class BaseCursor:
         self,
         query: str,
         parameters: Optional[Sequence[ParameterType]] = None,
-        set_parameters: Optional[Dict] = None,
         skip_parsing: bool = False,
-    ) -> int:
+        async_execution: Optional[bool] = False,
+    ) -> Union[int, str]:
         """Prepare and execute a database query.
 
         Supported features:
@@ -427,24 +510,26 @@ class BaseCursor:
             query (str): SQL query to execute
             parameters (Optional[Sequence[ParameterType]]): A sequence of substitution
                 parameters. Used to replace '?' placeholders inside a query with
-                actual values.
-            set_parameters (Optional[Dict]): List of set parameters to execute
-                a query with. DEPRECATED: Use SET SQL statements instead.
+                actual values
             skip_parsing (bool): Flag to disable query parsing. This will
-                disable parameterized, multi-statement, and SET queries,
-                while improving performance.
+                disable parameterized, multi-statement and SET queries,
+                while improving performance
+            async_execution (bool): flag to determine if query should be asynchronous
 
         Returns:
             int: Query row count.
         """
         params_list = [parameters] if parameters else []
-        await self._do_execute(query, params_list, set_parameters, skip_parsing)
-        return self.rowcount
+        await self._do_execute(query, params_list, skip_parsing, async_execution)
+        return self.query_id if async_execution else self.rowcount
 
     @check_not_closed
     async def executemany(
-        self, query: str, parameters_seq: Sequence[Sequence[ParameterType]]
-    ) -> int:
+        self,
+        query: str,
+        parameters_seq: Sequence[Sequence[ParameterType]],
+        async_execution: Optional[bool] = False,
+    ) -> Union[int, str]:
         """Prepare and execute a database query.
 
         Supports providing multiple substitution parameter sets, executing them
@@ -468,12 +553,17 @@ class BaseCursor:
                substitution parameter sets. Used to replace '?' placeholders inside a
                query with actual values from each set in a sequence. Resulting queries
                for each subset are executed sequentially.
+            async_execution (bool): flag to determine if query should be asynchronous
 
         Returns:
-            int: Query row count
+            int|str: Query row count for synchronous execution of queries,
+            query ID string for asynchronous execution.
         """
-        await self._do_execute(query, parameters_seq)
-        return self.rowcount
+        await self._do_execute(query, parameters_seq, async_execution=async_execution)
+        if async_execution:
+            return self.query_id
+        else:
+            return self.rowcount
 
     def _parse_row(self, row: List[RawColType]) -> List[ColType]:
         """Parse a single data row based on query column types."""
@@ -483,12 +573,10 @@ class BaseCursor:
         ]
 
     def _get_next_range(self, size: int) -> Tuple[int, int]:
-        cleandoc(
-            """
-            Return range of next rows of size (if possible),
-            and update _idx to point to the end of this range.
-            """
-        )
+        """
+        Return range of next rows of size (if possible),
+        and update _idx to point to the end of this range
+        """
 
         if self._rows is None:
             # No elements to take
@@ -540,6 +628,44 @@ class BaseCursor:
     def setoutputsize(self, size: int, column: Optional[int] = None) -> None:
         """Set a column buffer size for fetches of large columns (does nothing)."""
 
+    @check_not_closed
+    async def cancel(self, query_id: str) -> None:
+        """Cancel a server-side async query."""
+        await self._api_request(
+            parameters={"query_id": query_id},
+            path="cancel",
+            use_set_parameters=False,
+        )
+
+    @check_not_closed
+    async def get_status(self, query_id: str) -> QueryStatus:
+        """Get status of a server-side async query. Return the state of the query."""
+        try:
+            resp = await self._api_request(
+                # output_format must be empty for status to work correctly.
+                # And set parameters will cause 400 errors.
+                parameters={"query_id": query_id},
+                path="status",
+                use_set_parameters=False,
+            )
+            if resp.status_code == codes.BAD_REQUEST:
+                raise OperationalError(
+                    f"Asynchronous query {query_id} status check failed: "
+                    f"{resp.status_code}."
+                )
+            resp_json = resp.json()
+            if "status" not in resp_json:
+                raise OperationalError(
+                    f"Invalid response to asynchronous query: missing status."
+                )
+        except Exception:
+            self._state = CursorState.ERROR
+            raise
+        # Remember that query_id might be empty.
+        if resp_json["status"] == "":
+            return QueryStatus.NOT_READY
+        return QueryStatus[resp_json["status"]]
+
     # Context manager support
     @check_not_closed
     def __enter__(self) -> BaseCursor:
@@ -553,7 +679,7 @@ class BaseCursor:
 
 class Cursor(BaseCursor):
     """
-    Executes asyncio queries to Firebolt Database.
+    Executes async queries to Firebolt Database.
     Should not be created directly;
     use :py:func:`connection.cursor <firebolt.async_db.connection.Connection>`
 
@@ -577,43 +703,48 @@ class Cursor(BaseCursor):
         self,
         query: str,
         parameters: Optional[Sequence[ParameterType]] = None,
-        set_parameters: Optional[Dict] = None,
         skip_parsing: bool = False,
-    ) -> int:
+        async_execution: Optional[bool] = False,
+    ) -> Union[int, str]:
         async with self._async_query_lock.writer:
             return await super().execute(
-                query, parameters, set_parameters, skip_parsing
+                query, parameters, skip_parsing, async_execution
             )
 
     @wraps(BaseCursor.executemany)
     async def executemany(
-        self, query: str, parameters_seq: Sequence[Sequence[ParameterType]]
+        self,
+        query: str,
+        parameters_seq: Sequence[Sequence[ParameterType]],
+        async_execution: Optional[bool] = False,
     ) -> int:
+        """
+        Prepare and execute a database query against all parameter
+        sequences provided.
+        """
         async with self._async_query_lock.writer:
-            return await super().executemany(query, parameters_seq)
-        """
-            Prepare and execute a database query against all parameter
-            sequences provided.
-        """
+            return await super().executemany(query, parameters_seq, async_execution)
 
     @wraps(BaseCursor.fetchone)
     async def fetchone(self) -> Optional[List[ColType]]:
         async with self._async_query_lock.reader:
+            """Fetch the next row of a query result set."""
             return super().fetchone()
-        """Fetch the next row of a query result set."""
 
     @wraps(BaseCursor.fetchmany)
     async def fetchmany(self, size: Optional[int] = None) -> List[List[ColType]]:
         async with self._async_query_lock.reader:
+            """
+            Fetch the next set of rows of a query result;
+            size is cursor.arraysize by default.
+            """
             return super().fetchmany(size)
-        """Fetch the next set of rows of a query result;
-          size is cursor.arraysize by default."""
 
     @wraps(BaseCursor.fetchall)
     async def fetchall(self) -> List[List[ColType]]:
         async with self._async_query_lock.reader:
+            """Fetch all remaining rows of a query result."""
             return super().fetchall()
-        """Fetch all remaining rows of a query result."""
 
     @wraps(BaseCursor.nextset)
     async def nextset(self) -> None:
